<?php

/**
 * Localize script for cart data.
 */
add_action( 'wp_enqueue_scripts', function () {
	// ajax data for cart
	wp_localize_script( 'sage/main.js', 'bt_cart_data', [
		'ajax_url' => admin_url( 'admin-ajax.php' ),
		//'nonce' => wp_create_nonce( 'bt_cart_nonce' ),
	] );
}, 169);


/**
 * AJAX responder to set the cart quantity for any item.
 *
 * It echoes a string of JSON which can be used by JS to put content where it belongs.
 */
add_action( 'wp_ajax_bt_cart_update', 'bt_cart_update' );
add_action( 'wp_ajax_nopriv_bt_cart_update', 'bt_cart_update' );
function bt_cart_update() {
	// no nonce since caching seemed to cause issues with that elsewhere

    // make sure we have woocommerce
    if ( ! function_exists( 'WC' ) ) {
	    wp_die();
    }

	// init a result message (maybe just for debugging) and a notice message (something to send back to user if need be)
	$result = 'Ran cart update; no change (could be first load).';
	$notice = '';

	// make sure we have a product id and quantity value before we run cart updates
	if ( isset( $_REQUEST['product_id'] )
	     && is_numeric( $_REQUEST['product_id'] )
	     && isset( $_REQUEST['new_qty'] )
	     && is_numeric( $_REQUEST['new_qty'] )
	) {
		// set variation to 0 if there isn't one
		if ( ! isset( $_REQUEST['variation_id'] ) || ! is_numeric( $_REQUEST['variation_id'] ) ) {
			$_REQUEST['variation_id'] = 0;
		}

    // see if we are adding a donation, which has an input for amount
    $donation_amount = null;
    if ( isset( $_REQUEST['donation_amount'] ) && is_numeric( $_REQUEST['donation_amount'] ) && $_REQUEST['donation_amount'] > 5 ) {
      $donation_amount = $_REQUEST['donation_amount'];
    }

		// determine item key
    $cart_item_data = [];
    // -- if WooCommerce Subscriptions exists, we need to have some empty data to match (WON'T WORK FOR ADDING SUBSCRIPTION PRODUCTS THOUGH!)
    if ( class_exists( '\WC_Subscriptions' ) ) {
      $cart_item_data['wcsatt_data'] = [
        'active_subscription_scheme' => null,
      ];
    }
    // -- use donation amount as unique data too if it exists
    if ( $donation_amount ) {
      $cart_item_data['bt_donation_amount'] = $donation_amount;
    }
		$item_key = WC()->cart->generate_cart_id(
      $_REQUEST['product_id'],
      $_REQUEST['variation_id'],
      [],
      $cart_item_data
    );
		//error_log( __METHOD__ . ' -- $item_key = ' . $item_key );

		// if using sellers, and adding an item when something is already in the cart, ensure this item belongs to the
        // same seller as is already in the cart
		if ( bt_use_sellers() && $_REQUEST['new_qty'] > 0  && WC()->cart->get_cart_contents_count() > 0 ) {
			$seller_data = WC()->session->get( 'seller' );

			// get seller user id of the item being added to cart
			$new_item_user_id = get_post_field( 'post_author', $_REQUEST['product_id'] );

			// test if it's a different seller user, but only if we know seller id - we always should but adding this change in live sessions means those sessions may not
			if (
				isset( $seller_data['seller_user_id'] )
				&& $new_item_user_id != $seller_data['seller_user_id']
			) {
			    // seller mismatch!
				$notice = 'You have an order in progress for a different seller, so this item cannot be added.'
                          . "\n\n"
                          . 'Orders are for one seller at a time.';
				if ( empty( get_current_user_id() ) ) {
					$notice .= "\n\n" . 'Create an account during checkout to save your information for quick checkouts with additional sellers!';
                }
				// set new qty to zero which would remove item if in cart but is harmless if not
				$_REQUEST['new_qty'] = 0;
			}
		}

		//error_log( __METHOD__ . ' -- REQUEST = ' . print_r( $_REQUEST, true ) );

		// update cart
		if ( $_REQUEST['new_qty'] > 0 ) {

			// if product has a max qty, enforce this
			if ( $max_qty = get_field( 'max_per_order', $_REQUEST['product_id'] ) ) {
				// limit new qty to max per order
				//error_log( 'CART UPDATE: we have a max of ' . $max_qty . ' for product id ' . $_REQUEST['product_id'] );
				if ( $_REQUEST['new_qty'] > $max_qty ) {
					//error_log( 'CART UPDATE: hit the max! Requested ' . $_REQUEST['new_qty'] );
					$notice = 'There is a maximum of ' . $max_qty . ' per order for ' . addslashes( get_the_title( $_REQUEST['product_id'] ) ) . ', so we have limited your order quantity to this.';
					$_REQUEST['new_qty'] = $max_qty;
				}
			}

			// if product is one-of-a-kind, see if it's already in the cart
            if ( $one_of_a_kind = get_field( 'one_of_a_kind', $_REQUEST['product_id'] ) ) {
            	// look for existing item
	            if ( $item = WC()->cart->get_cart_item( $item_key ) ) {
	            	$notice = 'This one-of-a-kind item is already in your cart.';
		            $_REQUEST['new_qty'] = 1;
	            }
            }

			// check for product stock
			if ( $_REQUEST['variation_id'] > 0 ) {
				$product = new WC_Product_Variation( $_REQUEST['variation_id'] );
			} else {
				$product = new WC_Product( $_REQUEST['product_id'] );
			}
			if ( $product->managing_stock() ) {
				// stock is managed for this product, so we need to limit qty to in-stock amount
				/* Held isn't working??? Maybe that's ok, could get messy, and cart does enforce stock on checkout
				error_log( 'STOCK FOR ' . $product->get_title()
						   . ' IS ' . $product->get_stock_quantity()
						   . ' AND HELD IS ' . wc_get_held_stock_quantity( $product ) ); */
				$stock = $product->get_stock_quantity();
				if ( $_REQUEST['new_qty'] > $stock ) {
					//error_log( 'CART UPDATE: hit the stock amount! Requested ' . $_REQUEST['new_qty'] );
					$notice = 'There is currently only ' . $stock . ' in stock for ' . addslashes( get_the_title( $_REQUEST['product_id'] ) ) . ', so we have limited your order quantity to this.';
					$_REQUEST['new_qty'] = $stock;
				}
			}

      // look for existing item
			//error_log( __METHOD__ . ' -- WC()->cart->cart_contents KEYS = ' . print_r( array_keys( WC()->cart->cart_contents ), true ) );
			if ( $item = WC()->cart->get_cart_item( $item_key ) ) {
				// found it, change qty
				WC()->cart->set_quantity( $item_key, $_REQUEST['new_qty'] );
				$result = 'Updated ' . $_REQUEST['product_id'] . ', new qty is ' . $_REQUEST['new_qty'] . '.';

			} else {
				// not in cart already - add item to cart
                // @todo make sure this item is for the current seller, if applicable
				$add_to_cart_result = WC()->cart->add_to_cart( $_REQUEST['product_id'], $_REQUEST['new_qty'], $_REQUEST['variation_id'] );
				$result = 'Added ' . $_REQUEST['new_qty'] . ' of ' . $_REQUEST['product_id'] . ' to cart.';
        // set donation amount if applicable/exists
        if ( $donation_amount ) {
          //$item = WC()->cart->get_cart_item( $add_to_cart_result );
          //error_log( "ATC RESULT = $add_to_cart_result // GOING TO CHANGE DONATION AMOUNT TO $donation_amount FOR ITEM DATA: " . print_r( $item, true ) );
          WC()->session->set(
            'bt_donation',
            [
              $_REQUEST['product_id'] => $donation_amount,
            ]
          );
          // trigger calculation of totals so the filter that updates the item price runs
          WC()->cart->calculate_totals();
        }
			}

		} else if ( ! empty( $item_key ) ) {
			// zero - remove item
			WC()->cart->remove_cart_item( $item_key );
			$result = 'Removed ' . $_REQUEST['product_id'] . ' from cart.';
		}
	}

	// construct data to send back to js
	$item_count = WC()->cart->get_cart_contents_count();
	$data = [
		'result' => $result,
		'notice' => $notice,
		'updated_count' => $item_count,
		'updated_count_label' => 'item' . ( $item_count == 1 ? '' : 's' ),
		'updated_cost' => WC()->cart->get_cart_total(),
		'updated_item_qty' => $_REQUEST['new_qty'], // we may have modified this for max-per-order or stock limits
		'seller_id' => '',
		'seller_name' => '',
		'seller_logo' => '',
		'seller_url' => '',
		'seller_open_status' => '',
    'donation_amount' => $donation_amount,
	];

	//error_log( __METHOD__ . ' -- DATA FOR JS = ' . print_r( $data, true ) );

	// include seller data if there is anything in the cart
	if ( bt_use_sellers() ) {

        // take action based on item count
        if ( $item_count == 0 ) {
            // empty cart - clear seller
            WC()->session->set( 'seller', null );
            //error_log( 'CART UPDATE: cart is empty, so clearing seller' );
        } else {
            // something is in the cart - set seller data if we need to
	        $seller_data = WC()->session->get( 'seller' );
            if ( empty( $seller_data ) ) {
	            //error_log( 'CART UPDATE: cart has item, but seller is empty, so building up seller data...' );

	            // get seller post; unused but this will trigger determination of seller and save data to cart session
	            $seller = bt_get_cart_seller();

	            /* THIS IS DONE BY bt_get_cart_seller() NOW
				// set seller data
				$seller_data['seller_id'] = $seller->ID;
	            // [SNIP]
				// save this seller data to cart session
				WC()->session->set( 'seller', $seller_data );
				*/

                // get the seller data from the cart session so we have it ready to go in our return data
	            $seller_data = WC()->session->get( 'seller' );

            //} else {
	            //error_log( 'CART UPDATE: cart has item, and we already have seller data: ' . print_r( $seller_data, 1 ) );
            }

	        // get open status via locations (we get this fresh all the time so it's current to the last cart action)
	        if ( $location_slug = bt_use_sellers( 'seller_location_slug' ) ) {

                if ( $seller_data['seller_does_curbside'] && $seller_data['seller_does_curbside_by_appointment_only'] ) {
	                // seller is by appointment only
	                $seller_data['seller_open_status'] = '';
	                $seller_data['seller_next_open_day'] = 'by appointment';
	                $seller_data['seller_no_curbside'] = '';

                } else if ( $seller_data['seller_does_usps'] && ! $seller_data['seller_does_curbside'] ) {
                    // seller only ships usps, doesn't do curbside
	                $seller_data['seller_open_status'] = '';
	                $seller_data['seller_next_open_day'] = '';
	                $seller_data['seller_no_curbside'] = 'For Delivery';

                } else {
                    // curbside is an option and hours should be involved
	                $locations = new \App\Locations( $seller_data['seller_id'] );
	                $seller_data['seller_open_status'] = $locations->{$location_slug.'Status'}();
	                $seller_data['seller_next_open_day'] = $locations->{$location_slug.'NextOpenDay'}();
	                $seller_data['seller_no_curbside'] = '';
                }
	        }

	        // include the seller data in returned data
            foreach ( $seller_data as $key => $value ) {
                $data[$key] = $value;
            }
        }

	}

	// return the data as JSON string
	echo json_encode( $data );
	wp_die();
};


/**
 * Hook into before cart table, to add information about the seller.
 */
add_action( 'woocommerce_before_cart_table', 'bt_cart_seller_header' );


/**
 * Hook into cart contents, to add a table row for a Tip.
 */
add_action( 'woocommerce_cart_contents', function() {
    if ( bt_use_tips() ) {
        ?>
        <tr>
            <td colspan="6" data-title="Tip">
                <div class="d-flex align-items-center justify-content-end">
                    <strong class="mr-3">Add a Tip</strong>
                    <span class="mr-1">
                        <input id="cart-tip" type="number" size="2" min="0" max="1000" name="tip_amount" value="<?= WC()->session->get( 'tip_amount' ) ?>"> %
                    </span>
                </div>
            </td>
        </tr>
        <?php
    }
} );


/**
 * Hook into cart fee calculation, to add a fee for Tip.
 */
add_action( 'woocommerce_cart_calculate_fees', function () {
	//error_log( 'CALC FEES: starting...' );
	global $woocommerce;

	if ( is_admin() && ! defined( 'DOING_AJAX' ) )
		return;

	// process tips if used
	if ( bt_use_tips() ) {
		// init to seller preference if not set
		if ( ! is_numeric( WC()->session->get( 'tip_amount' ) ) ) {
			//error_log( 'CALC FEES: tip is EMPTY: ' . WC()->session->get( 'tip_amount' ) );
			$seller      = bt_get_cart_seller();
			$default_tip = get_field( 'default_tip', $seller );
			if ( ! is_numeric( $default_tip ) ) {
				// @todo Pull this from ACF field default value
				$default_tip = 0;
			}
			WC()->session->set( 'tip_amount', $default_tip );
			//error_log( 'CALC FEES: set DEFAULT tip to ' . $default_tip );
			//} else {
			//error_log( 'CALC FEES: not empty, is ' . WC()->session->get( 'tip_amount' ) . '. If zero, was it on purpose?' );
		}

		//error_log( 'CART FEES. POST = ' . print_r($_POST,1) );
		if ( isset( $_POST['tip_amount'] ) && is_numeric( $_POST['tip_amount'] ) && $_POST['tip_amount'] >= 0 ) {
			//error_log( 'CALC FEES: set tip to POST VAL ' . $_POST['tip_amount'] );
			// Set session and save data
			WC()->session->set( 'tip_amount', $_POST['tip_amount'] );
		}

		// calculate percentage for tip and save the fee
		$percentage = WC()->session->get( 'tip_amount' ) / 100;
		$tip        = $woocommerce->cart->cart_contents_total * $percentage;
		$woocommerce->cart->add_fee( WC()->session->get( 'tip_amount' ) . '% Tip', $tip, false );
	}
} );


/**
 * Hook into add to cart action to:
 * - enforce max quantity
 *
 * Note: the item is already added to the cart when this runs.
 */
add_action('woocommerce_add_to_cart', function() {
	//error_log( 'ADD TO CART: ' . print_r($_REQUEST,1) );
	//error_log( 'CART: ' . print_r(WC()->cart->get_cart(),1) );

	// check product ID; might always be the first case but not sure
	if ( isset( $_REQUEST['add-to-cart'] ) ) {
		//error_log(' -- FROM ADD-TO-CART');
		$product_id = $_REQUEST['add-to-cart'];
	} else {
		//error_log(' -- FROM PRODUCT_ID');
		$product_id = $_REQUEST['product_id'];
	}

	// check product to see if it has a max; we only do anything here if so
	if ( $max_qty = get_field( 'max_per_order', $product_id ) ) {
		//error_log(' -- MAX QTY = ' . $max_qty);

		// set empty variation if need be
		if ( isset( $_REQUEST['variation_id'] ) ) {
			$variation_id = $_REQUEST['variation_id'];
		} else {
			$variation_id = null;
		}

		// get the item key so we can get it from the cart
		$item_key = WC()->cart->generate_cart_id( $product_id, $variation_id );

		// get current cart item
		if ( $item = WC()->cart->get_cart_item( $item_key ) ) {
			// item already exists - check current quantity
			if ( $item['quantity'] > $max_qty ) {
				// over limit - set to max
				WC()->cart->set_quantity( $item_key, $max_qty );
				wc_add_notice( 'You can only order up to ' . $max_qty . ' of ' . get_the_title( $product_id ) . ', so your quantity has been adjusted to the maximum.', 'error' );
			}
		}
	}
} );


/**
 * Hook into update cart item quantity action to:
 * - enforce max quantity
 */
add_action( 'woocommerce_after_cart_item_quantity_update', function ( $item_key, $quantity, $old_quantity, $cart ){
	// get the item so we have product/post data
	$item = $cart->get_cart_item( $item_key );
	// check for a max quantity; only do anything if there is a max
	if ( $max_qty = get_field( 'max_per_order', $item['product_id'] ) ) {
		// we have a max; see if we're over it
		if ( $quantity > $max_qty ) {
			// over max; set quantity to the max
			WC()->cart->set_quantity( $item_key, $max_qty );
			wc_add_notice( 'You can only order up to ' . $max_qty . ' of ' . get_the_title( $item['product_id'] ) . ', so your quantity has been adjusted to the maximum.', 'error' );
		}
	}
}, 20, 4 );


/**
 * Hook into total calculation to:
 * - set custom donation amount
 */
add_action( 'woocommerce_before_calculate_totals', function( $cart ) {
  // check if there is any donation data
  $donation_data = WC()->session->get( 'bt_donation' );
  if ( ! empty( $donation_data ) ) {
    // check all cart items
    foreach ( $cart->get_cart() as $hash => $value ) {
      if ( array_key_exists( $value['product_id'], $donation_data ) ) {
        // we have a donation amount for this product ID, so set the price to that amount
        $new_price = $donation_data[ $value['product_id'] ];
        $value[ 'data' ]->set_price( $new_price );
        // update the item name too
        if ( ! $new_name = bt_ecomm_option( 'donation_cart_item_prefix' ) ) {
          $new_name = 'Support Us:';
        }
        $new_name .= " $$new_price Donation";
        $value[ 'data' ]->set_name( $new_name );
      }
    }
  }
}, 1, 9987 );

